<?php
/*
     RemoteSearch Mediawiki extension. Search relational server.
     See more RemoteSearch.setup.php
     See detail http://metabolomics.jp/wiki/Help:Extension/RemoteSearch

     *********************************************************************

     Copyright (C) 2009-2010 Metabolomics.JP Development Team

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

     *********************************************************************
     Author: Arita laboratory, with Kazuhiro Suwa <metabolome@cb.k.u-tokyo.ac.jp>
     URL: http://metabolomics.jp/
*/

require_once( "HTTP/Request.php" );

class RemoteSearch extends SpecialPage
{
	var $prefix = "rs_";

	function RemoteSearch() {
		SpecialPage::SpecialPage("RemoteSearch");
		self::loadMessages();
	}
 
	function execute( $par ) {
		global $wgRequest, $wgOut, $wgParser;

		# access control

		# preprocess
		$wgOut->disable();
		wfResetOutputBuffers();
		header( "Content-type: text/plain; charset=utf-8" );

		# output data
		$word = $wgRequest->getText( $this->prefix . "word" );
		$ns   = $wgRequest->getText( $this->prefix . "ns" );
		$page = $wgRequest->getText( $this->prefix . "page" );
		$flag = $wgRequest->getText( $this->prefix . "search" );
		if( strlen( $word ) > 0 ){
			if( strlen( $ns ) == 0 )
				$ns = 'Main';

			$res = "";
			$dp = new DummyParser( Title::makeTitle( -1, 'RemoteSearch' ) );
			$exFunctions = new ExFunctions;
			if( strcmp( 'title', $flag ) == 0 ){
				$res = $exFunctions->searchTitle( $dp, $word, $ns );
			} else if( strcmp( 'line', $flag ) == 0 ){
				$res = $exFunctions->searchLine( $dp, $word, $ns, $page );
			}
			print $res;
		}
	}

	function loadMessages() {
		static $messagesLoaded = false;
		global $wgMessageCache;
		if ( !$messagesLoaded ) {
			$messagesLoaded = true;
 
			require( dirname( __FILE__ ) . '/RemoteSearch.i18n.php' );
			foreach ( $allMessages as $lang => $langMessages ) {
				$wgMessageCache->addMessages( $langMessages, $lang );
			}
		}
		return true;
	}

	function remoteSearchLine( &$parser, $urls, $word, $namespace = 'Main', $page = '' ) {
		$res = $this->httpAccess( $parser, $urls, $word, $namespace, $page, 'line' );
		return $res;
	}

	function remoteSearchTitle( &$parser, $urls, $word, $namespace = 'Main' ) {
		$res = $this->httpAccess( $parser, $urls, $word, $namespace, '', 'title' );
		return $res;
	}

	var $option = array(
		"timeout" => "3",
		"allowRedirects" => false,
		"maxRedirects" => 0,
	);
	function httpAccess( &$parser, $urls, $word, $namespace, $page, $flag ){
		global $wgSitename, $egRemoteSearchURL, $wgServerName;

		$exFunctions = new ExFunctions();
		$url = split( ':', $urls );
		$res = "";
		$own = $wgServerName . "/";
		$ownlength = mb_strlen( $own );
		for( $i = 0; $i < count( $url ); $i ++ ){
			if( !isset( $egRemoteSearchURL[$url[$i]] ) )
				continue;

			if( strncmp( $own, $egRemoteSearchURL[$url[$i]], $ownlength ) == 0 ){
				// self server
				if( strcmp( $flag, "title" ) == 0 )
					$res .= $exFunctions->searchtitle( $parser, $word, $namespace );
				else if( strcmp( $flag, "line" ) == 0 )
					$res .= $exFunctions->searchLine( $parser, $word, $namespace, $page );

			} else if( strlen( $egRemoteSearchURL[$url[$i]] ) > 0 ) {
				$req = "http://" . $egRemoteSearchURL[$url[$i]];
				if( ereg( "[^/]$", $req ) )
					$req .= "/";
				$req .= "Special:RemoteSearch?" . $this->prefix . "word=$word&" . $this->prefix . "ns=$namespace&" . $this->prefix . "page=$page&" . $this->prefix . "search=$flag";

				$http = new HTTP_Request( $req, $this->option );
				$http->addHeader( "User-Agent", "$wgSitename" );
				$http->addHeader( "Referer", "-" );
				$result = $http->sendRequest();
				if( !PEAR::isError( $result ) ) {
					if( $http->getResponseCode() < 400) {
						$res .= $http->getResponseBody();
					}
				}
			}
		}

		return $res;
	}
}

class DummyParser {
	var $mTitle;

	function __construct( $title ) {
		$this->mTitle = $title;
	}

	function getTitle() { return $this->mTitle; }
}
